/* ****************************************************************************
repairs daggerfall items -- slow, but careful version -- 12/20/96

Comments and constructive critism to:
	Tim Kieschnick
	`timk@tamu.edu'

**************************************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <ctype.h>

#define SLOT_MAX 5
#define SAVE_PREFIX "save"
#define SAVETREE "savetree.dat"
#define SAVETREE_BAK "savetree.new"
#define DF_OBJ_TOKEN_LEN 8

#ifdef __MSDOS__
#define DIR_SEP "\\"
#else
#define DIR_SEP "/"
#endif

/* this is the daggerfall item structure as best I can gather (???) */
typedef struct {
	char token[DF_OBJ_TOKEN_LEN];
	char name[32];
	char group[4];
	char type[4];
	char cost[4];
	unsigned int present:16, highest:16;
} df_object;

/* is string printable? */
int isstrprt(const char * s) {
	unsigned int i;

	for (i = 0; i < strlen(s); ++i) {
		if (!isprint(s[i])) return 0;
	}
	return 1;
}

/* bytewise comparison */
int bytecmp(const char *s1, const char *s2, unsigned int len) {
	unsigned int i;

	for (i=0; i<len; ++i) {
		if (s1[i] != s2[i]) return(s1[i] < s2[i] ? -1 : 1);
	}
	return(0);
}

int main(int argc, char *argv[]) {
	unsigned int slot;
	char *infile, *outfile;
	FILE *in, *out;
	struct stat *instat;
	unsigned long int i;
	df_object *ob;

	/* daggerfall object token separator  */
	char token[DF_OBJ_TOKEN_LEN];
	token[0] = 0x00;
	token[1] = 0x00;
	token[2] = 0x00;
	token[3] = 0x00;
	token[4] = 0x34;
	token[5] = 0x00;
	token[6] = 0x00;
	token[7] = 0x00;

	/* checking arguments */
	if (argc < 2 || (slot = (unsigned int) atoi(argv[1])) > SLOT_MAX) {
		printf("Daggerfall Repair Objects Utility\n");
		printf("Usage: %s #slot\n\twhere #slot is int between 0 and %d.\n", argv[0], SLOT_MAX);
		return(1);
	}

	/* making input and output filename strings */
	if (NULL == (infile = (char *) calloc(strlen(SAVE_PREFIX) + 1 + strlen(DIR_SEP) + strlen(SAVETREE) + 1, sizeof(char)))) {
		printf("Not enough memory to allocate space for input file name.\n");
		return(2);
	}
	sprintf(infile, "%s%u%s%s", SAVE_PREFIX, slot, DIR_SEP, SAVETREE);

	if (NULL == (outfile = (char *) calloc(strlen(SAVE_PREFIX) + 1 + strlen(DIR_SEP) + strlen(SAVETREE_BAK) + 1, sizeof(char)))) {
		printf("Not enough memory to allocate space for output file name.\n");
		return(2);
	}
	sprintf(outfile, "%s%u%s%s", SAVE_PREFIX, slot, DIR_SEP, SAVETREE_BAK);

	#ifdef DEBUG
		printf("Using `%s' for input.\nUsing `%s' for output.\n", infile, outfile);
	#endif

	printf("Repairing objects in `save%u'.\n", slot);

	if (NULL == (in = fopen(infile, "rb"))) {
		perror(infile);
		return(3);
	}

	if (NULL == (out = fopen(outfile, "wb"))) {
		perror(outfile);
		return(3);
	}

	if (NULL == (instat = (struct stat *) malloc(sizeof(struct stat)))) {
		printf("Not enough memory to allocate %u bytes.\n", sizeof(struct stat));
		return(2);
	}

	if (NULL == (ob = (df_object *) malloc(sizeof(df_object)))) {
		printf("Not enough memory to allocate %u bytes.\n", sizeof(df_object));
		return(2);
	}

	if (stat(infile, instat)) {
		perror(infile);
		return(2);
	}

	#ifdef DEBUG
		printf("Savetree is %ld bytes.\n", instat->st_size);
	#endif

	i = 0;
	while (i < instat->st_size - sizeof(df_object)) {
		fread(ob, 1, sizeof(df_object), in);

		if (!(bytecmp(ob->token, token, DF_OBJ_TOKEN_LEN)) &&
			strlen(ob->name) > 0 && isstrprt(ob->name) &&
			ob->present < ob->highest) {

				printf("fixed %32s from %6d to %6d\n", ob->name, ob->present, ob->highest);
				ob->present = ob->highest;
				i += fwrite(ob, 1, sizeof(df_object), out);

			} else {
				i += fwrite(ob, 1, 1, out);
				fseek(in, i, SEEK_SET);
		}
	}


	fread(ob, instat->st_size - i, 1, in);
	i += fwrite(ob, 1, instat->st_size - i, out);

	fflush(out);

	fclose(in);
	fclose(out);

	if (i != instat->st_size) {
		printf("Error.\n");
		return(4);
	}

	printf("Done!\n");
	return(0);
}

/* dfrepobj.c */
